package de.hdm_stuttgart.mi.exam.unitmarking;

import java.util.HashMap;
import java.util.Map;

import org.junit.runner.Description;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunListener;

public class MarkingListener extends RunListener {

   private Map<String, Values> valuesByClassname = new HashMap<>();
   
   class Values {
      
      int points = 0,
          reachablePoints = 0,
          currentPoints;

      public int getPoints() {
         return points;
      }

      public int getReachablePoints() {
         return reachablePoints;
      }
   }

   @Override
   public void testFinished(final Description description) throws Exception {
      final Values values = addDescription(description);
      values.points += values.currentPoints;
      values.reachablePoints += values.currentPoints;
   }

   @Override
   public void testFailure(final Failure failure) throws Exception {
      final Values values = addDescription(failure.getDescription());
      // Ugly: testFinished will be called in any case. So we defeat
      // adding reached points in this method.
      values.points -= values.currentPoints;
   }

   Values addDescription(final Description description) {
      // Split fully qualified classname string
      final String[] components = description.getClassName().split("\\.");
      
      final String testClassName = components[components.length - 1];
      Values values = valuesByClassname.get(testClassName);
      if (null == values) {
         values = new Values();
         valuesByClassname.put(testClassName, values);
      }
      final Marking marking = description.getAnnotation(Marking.class);
      if (null == marking) {
         values.currentPoints = 1;
      } else {
         values.currentPoints = marking.points();
      }
      return values;
   }

   
   public String getPointSumms() {
      final StringBuffer ret = new StringBuffer();
      
      int reachablePointSum = 0,
            pointSum = 0;
      
      for (Map.Entry<String, Values> entry: valuesByClassname.entrySet()) {
         ret.append(entry.getKey());
         ret.append(": ");
         final Values value = entry.getValue();
         pointSum += value.points;
         ret.append(value.points);
         ret.append('/');
         ret.append(value.reachablePoints);
         reachablePointSum += value.reachablePoints;
         ret.append("  ");
      }
      ret.append("\nSum (" + pointSum + "/" + reachablePointSum + ")");
      return ret.toString();
   }
//      @Override
//      public void testRunStarted(Description description) throws Exception {
//         super.testRunStarted(description);
//         System.out.println("testRunStarted" + description);
//      }
   //
   //   @Override
   //   public void testRunFinished(Result result) throws Exception {
   //      super.testRunFinished(result);
   //      System.out.println("testRunFinished");
   //   }
   //
   //   @Override
   //   public void testStarted(Description description) throws Exception {
   //      super.testStarted(description);
   //      System.out.println("testStarted");
   //   }
   //
   //   @Override
   //   public void testAssumptionFailure(Failure failure) {
   //      super.testAssumptionFailure(failure);
   //      System.out.println("testAssumptionFailure");
   //   }
   //
   //   @Override
   //   public void testIgnored(Description description) throws Exception {
   //      super.testIgnored(description);
   //      System.out.println("testIgnored");
   //   }   
}
